'use strict';
var _ = require('underscore');
var joi = require('joi');
var Foxx = require('org/arangodb/foxx');
var ArangoError = require('org/arangodb').ArangoError;
var AttachmentRepo = require('repositories/attachment');
var Attachment = require('models/attachment');
var controller = new Foxx.Controller(applicationContext);
var fs = require("fs");
var actions = require("org/arangodb/actions");

var directory = applicationContext.foxxFilename("attachments");
var nameGenerator = require("internal").genRandomAlphaNumbers;

var generateName = function () {
	while (true) {
		var name = nameGenerator(10);
		var filename = fs.join(directory, name);
		if (! fs.isFile(filename)) {
			// found a not-yet used filename
			return name;
		}
	}
};

var attachmentIdSchema = joi.string().required()
.description('The id of the attachment')
.meta({allowMultiple: false});

var attachmentRepo = new AttachmentRepo(
  applicationContext.collection('attachment'),
  {model: Attachment}
);

/**
 * Returns a list of all files
 *
 * The response will be a list of files, wrapped in simple HTML. 
 */
controller.get('/list', function (req, res) {
  res.json(_.map(attachmentRepo.all(), function (model) {
    return model.forClient();
  }));
});

/**
 * Stores a file on the server
 *
 * The request has to be of type multipart/form-data. The first part of the request has to contain
 * the "form" data (parameters "description* and *file*). The second part of the request has to contain
 * the actual file data in binary format.
 */
controller.post('/store', function (req, res) {
	var filename = generateName();
	var description, data, contentType, originalName;
	var parts = req.requestParts();

	parts.forEach(function (part, i) {
		if (i === 0) {
			description = part.data.toString('utf-8');
		}
		else {
			data = part.data;
			var cd = part.headers["Content-Disposition"];
			if (cd) {
				var pos = cd.indexOf("filename=\"");
				var len = "filename=\"".length;
				if (pos !== -1) {
					originalName = cd.substring(pos + len, cd.length - 1);
				}
				contentType = part.headers["Content-Type"];
			}
		}
	});

	var datum = new Date();
	var pfad = datum.getFullYear() * 100 + datum.getMonth() + 1;
	if (!fs.isDirectory(fs.join(directory, pfad.toString())))
		fs.makeDirectory(fs.join(directory, pfad.toString()));
	filename = fs.join(pfad.toString(), filename);
	
	fs.write(fs.safeJoin(directory, filename), data);
	var erg = null;
	try {
		var doc = { 
			name: filename.replace(fs.pathSeparator, '_'), 
			size: data.length
		};
		if (description !== undefined) {
			doc.description = description;
		}
		if (contentType !== undefined) {
			doc.contentType = contentType;
		}
		if (originalName !== undefined) {
			doc.originalName = originalName;
		} 
		erg = attachmentRepo.save(doc);
	}
	catch (err) {
		// must remove from the filesystem too
		fs.remove(filename);
		throw err;
	}

	res.send(erg.forClient());
})
.errorResponse(
	"Error", actions.HTTP_BAD, "No file uploaded or description missing", function(e) {
		return {
			error: true,
			code: actions.HTTP_BAD,
			errorMessage: "No file uploaded or description missing"
		};
	}
);

/**
 * Returns the contents of the specified file
 * 
 * The response body will contain the file contents in a binary format.
 */
controller.get('/fetch/:filename', function (req, res) {
	var name = req.params("filename");
	var doc = attachmentRepo.firstExample({ name: name });

	if (! doc) {
		throw new ArangoError();
	}
	
	if(name.indexOf('_') > 0)
		name = name.replace('_', fs.pathSeparator);
	var filename = fs.safeJoin(directory, name);

	if (! fs.isFile(filename)) {
		throw new ArangoError();
	}

	if (doc.originalName) {
		res.set("Content-Disposition", "attachment; filename=\"" + doc.originalName + "\"");
	}
	if (doc.contentType) {
		res.set("Content-Type", doc.contentType);
	}
	res.sendFile(filename);
})
.pathParam("filename", {
	type: joi.string().required()
})
.errorResponse(
	Error, actions.HTTP_NOT_FOUND, "The requested file could not be found", function(e) {
		return {
			error: true,
			code: actions.HTTP_NOT_FOUND,
			errorMessage: "The requested file could not be found"
		};
	}
);

/** Lists of all issue.
 *
 * This function simply returns the list of all Issue.
 */
controller.get('/', function (req, res) {
	var filter = req.params('filter');
	console.log('Filter: ', filter);
	if(filter) {
		var lst = null;
		lst = Foxx.createQuery('for x in ' + applicationContext.mount.substring(1) + '_attachment filter ' + filter + ' return x')();
		if(lst !== null) {
			res.json(lst);
			return;
		}
	}
	res.json(_.map(attachment.all(), function (model) {
		return model.forClient();
	}));
})
.queryParam('filter', {
  description: 'Filter for attachment list',
  type: joi.string().optional()
});

/** Reads a attachment.
 *
 * Reads a attachment.
 */
controller.get('/:id', function (req, res) {
  var id = req.urlParameters.id;
  res.json(attachmentRepo.byId(id).forClient());
})
.pathParam('id', attachmentIdSchema)
.errorResponse(ArangoError, 404, 'The attachment could not be found');

/** Replaces a attachment.
 *
 * Changes a attachment. The information has to be in the
 * requestBody.
 */
controller.put('/:id', function (req, res) {
  var id = req.urlParameters.id;
  var attachment = req.parameters.attachment;
  res.json(attachmentRepo.replaceById(id, attachment));
})
.pathParam('id', attachmentIdSchema)
.bodyParam('attachment', {
  description: 'The attachment you want your old one to be replaced with',
  type: Attachment
})
.errorResponse(ArangoError, 404, 'The attachment could not be found');

/** Updates a attachment.
 *
 * Changes a attachment. The information has to be in the
 * requestBody.
 */
controller.patch('/:id', function (req, res) {
  var id = req.urlParameters.id;
  var patchData = req.parameters.patch;
  res.json(attachmentRepo.updateById(id, patchData));
})
.pathParam('id', attachmentIdSchema)
.bodyParam('patch', {
  description: 'The patch data you want your attachment to be updated with',
  type: joi.object().required()
})
.errorResponse(ArangoError, 404, 'The attachment could not be found');

/** Removes a attachment.
 *
 * Removes a attachment.
 */
controller.delete('/:id', function (req, res) {
  var id = req.urlParameters.id;
  var doc = attachmentRepo.byId(id);
  fs.remove(doc.get("filename"));
  attachmentRepo.removeById(id);
  res.json({success: true});
})
.pathParam('id', attachmentIdSchema)
.errorResponse(ArangoError, 404, 'The attachment could not be found');
