'use strict';
var _ = require('underscore');
var joi = require('joi');
var Foxx = require('org/arangodb/foxx');
var ArangoError = require('org/arangodb').ArangoError;
var CategoryRepo = require('repositories/category');
var Category = require('models/category');
var controller = new Foxx.Controller(applicationContext);

var categoryIdSchema = joi.string().required()
.description('The id of the category')
.meta({allowMultiple: false});

var categoryRepo = new CategoryRepo(
  applicationContext.collection('category'),
  {model: Category}
);

/** Lists of all category.
 *
 * This function simply returns the list of all Category.
 */
controller.get('/', function (req, res) {
  res.json(_.map(categoryRepo.all(), function (model) {
    return model.forClient();
  }));
});

/** Creates a new category.
 *
 * Creates a new category. The information has to be in the
 * requestBody.
 */
controller.post('/', function (req, res) {
  var category = req.parameters.category;
  res.json(categoryRepo.save(category).forClient());
})
.bodyParam('category', {
  description: 'The category you want to create',
  type: Category
});

/** Reads a category.
 *
 * Reads a category.
 */
controller.get('/:id', function (req, res) {
  var id = req.urlParameters.id;
  res.json(categoryRepo.byId(id).forClient());
})
.pathParam('id', categoryIdSchema)
.errorResponse(ArangoError, 404, 'The category could not be found');

/** Replaces a category.
 *
 * Changes a category. The information has to be in the
 * requestBody.
 */
controller.put('/:id', function (req, res) {
  var id = req.urlParameters.id;
  var category = req.parameters.category;
  res.json(categoryRepo.replaceById(id, category));
})
.pathParam('id', categoryIdSchema)
.bodyParam('category', {
  description: 'The category you want your old one to be replaced with',
  type: Category
})
.errorResponse(ArangoError, 404, 'The category could not be found');

/** Updates a category.
 *
 * Changes a category. The information has to be in the
 * requestBody.
 */
controller.patch('/:id', function (req, res) {
  var id = req.urlParameters.id;
  var patchData = req.parameters.patch;
  res.json(categoryRepo.updateById(id, patchData));
})
.pathParam('id', categoryIdSchema)
.bodyParam('patch', {
  description: 'The patch data you want your category to be updated with',
  type: joi.object().required()
})
.errorResponse(ArangoError, 404, 'The category could not be found');

/** Removes a category.
 *
 * Removes a category.
 */
controller.delete('/:id', function (req, res) {
  var id = req.urlParameters.id;
  categoryRepo.removeById(id);
  res.json({success: true});
})
.pathParam('id', categoryIdSchema)
.errorResponse(ArangoError, 404, 'The category could not be found');

/** Reads statistic of category.
 *
 * Reads statistic of category.
 */
controller.get('/stat/:id', function (req, res) {
  var id = req.urlParameters.id;
  var q = Foxx.createQuery('for x in ' + applicationContext.mount.substring(1) + 
  	'_issue filter x.category==@id collect status=x.status with count into anz return {status: status, count: anz}');
  var lst = q({id: id});
  res.json(lst);
})
.pathParam('id', categoryIdSchema)
.errorResponse(ArangoError, 404, 'The category statistic could not be found');
