'use strict';
var _ = require('underscore');
var joi = require('joi');
var Foxx = require('org/arangodb/foxx');
var ArangoError = require('org/arangodb').ArangoError;
var ProjectRepo = require('repositories/project');
var Project = require('models/project');
var controller = new Foxx.Controller(applicationContext);

var projectIdSchema = joi.string().required()
.description('The id of the project')
.meta({allowMultiple: false});

var projectRepo = new ProjectRepo(
  applicationContext.collection('project'),
  {model: Project}
);

/** Lists of all project.
 *
 * This function simply returns the list of all Project.
 */
controller.get('/', function (req, res) {
  res.json(_.map(projectRepo.all(), function (model) {
    return model.forClient();
  }));
});

/** Creates a new project.
 *
 * Creates a new project. The information has to be in the
 * requestBody.
 */
controller.post('/', function (req, res) {
  var project = req.parameters.project;
  res.json(projectRepo.save(project).forClient());
})
.bodyParam('project', {
  description: 'The project you want to create',
  type: Project
});

/** Reads a project.
 *
 * Reads a project.
 */
controller.get('/:id', function (req, res) {
  var id = req.urlParameters.id;
  res.json(projectRepo.byId(id).forClient());
})
.pathParam('id', projectIdSchema)
.errorResponse(ArangoError, 404, 'The project could not be found');

/** Replaces a project.
 *
 * Changes a project. The information has to be in the
 * requestBody.
 */
controller.put('/:id', function (req, res) {
  var id = req.urlParameters.id;
  var project = req.parameters.project;
  res.json(projectRepo.replaceById(id, project));
})
.pathParam('id', projectIdSchema)
.bodyParam('project', {
  description: 'The project you want your old one to be replaced with',
  type: Project
})
.errorResponse(ArangoError, 404, 'The project could not be found');

/** Updates a project.
 *
 * Changes a project. The information has to be in the
 * requestBody.
 */
controller.patch('/:id', function (req, res) {
  var id = req.urlParameters.id;
  var patchData = req.parameters.patch;
  res.json(projectRepo.updateById(id, patchData));
})
.pathParam('id', projectIdSchema)
.bodyParam('patch', {
  description: 'The patch data you want your project to be updated with',
  type: joi.object().required()
})
.errorResponse(ArangoError, 404, 'The project could not be found');

/** Removes a project.
 *
 * Removes a project.
 */
controller.delete('/:id', function (req, res) {
  var id = req.urlParameters.id;
  projectRepo.removeById(id);
  res.json({success: true});
})
.pathParam('id', projectIdSchema)
.errorResponse(ArangoError, 404, 'The project could not be found');

/** Reads statistic of project.
 *
 * Reads statistic of project.
 */
controller.get('/stat/:id', function (req, res) {
  var id = req.urlParameters.id;
  var q = Foxx.createQuery('for x in ' +applicationContext.mount.substring(1) + 
  	'_issue filter x.project==@id collect status=x.status with count into anz return {status: status, count: anz}');
  var lst = q({id: id});
  res.json(lst);
})
.pathParam('id', projectIdSchema)
.errorResponse(ArangoError, 404, 'The project statistic could not be found');

